import os.path
from typing import Dict, List, Optional
import configparser
from configparser import SectionProxy, ConfigParser

from .errors import NoPackagesInPipfileError, PipfilePathDoesNotExistError
from .operator_enums import OperatorEnum


# todo - Store this as a Github gist or re-usable, cross-repo package that can be used across Monolith projects

class RequirementsManager:
    """
    Manages reading the Pipfile and converting it into a correctly formatted list of strings for use in the
    'install_requires' and 'extra_requires' args of the setup process.

    Attributes:
        self.config: (Dict) list of raw line by line output of Pipfile
        self.packages: (Optional[Dict]) all packages and versions excl. dev-packages
        self.dev_packages: (Optional[Dict]) all dev-packages
    """
    def __init__(self, pipfile_loc: str = "./Pipfile") -> None:
        """
        :param pipfile_loc: (str) location of Pipfile
        :return: None
        :raises: (PipfilePathDoesNotExistError) if Pipfile path does not exist
        """
        if not os.path.exists(pipfile_loc):
            raise PipfilePathDoesNotExistError()
        self.config: ConfigParser = configparser.ConfigParser()
        self.config.read(pipfile_loc)
        self.packages: Optional[Dict] = None
        self.dev_packages: Optional[Dict] = None
        self._set_package_dicts()

    @staticmethod
    def _simplify_section(section: SectionProxy) -> Dict:
        """
        Parses each line of section from self.config Pipfile into dict with the following format:
        {<package_name>: >version>,...}
        :param section: (SectionProxy) proxy of config file section data
        :return: (Dict) key = package name, value = version
        """
        def _simplify_value(key):
            value = eval(section[key].replace("{", "dict(").replace("}", ")"))
            return '%s%s' % (key, section[key].strip('"')) if isinstance(value, str) \
                else '%s[%s]%s' % (key, value['extras'][0], value['version'])
        package_list = list(map(_simplify_value, section))
        return {package.split("==")[0]: package.split("==")[1] for package in package_list}

    def _set_package_dicts(self) -> None:
        """
        Sets the packages and dev_packages attributes from their sections in the self.config.
        :return: None
        :raises: (NoPackagesInPipfileError) If not packages found in self.config 'packages'
        """
        self.packages = self._simplify_section(self.config["packages"])
        if self.packages == {}:
            raise NoPackagesInPipfileError()
        self.dev_packages = self._simplify_section(self.config["dev-packages"])

    def get_packages(self, operator: OperatorEnum, extras_require: Optional[List[str]] = None) -> List[str]:
        """
        Get list of packages with correct version and given operator.
        :param operator: (OperatorEnum) e.g. '==' or '>='
        :param extras_require: (Optional[List[str]]) list of package names to exclusively return with version
        :return: (List[str]) e.g. [package==version,...]
        """
        if extras_require:
            return [f"{package}{operator.value}{version}" for package, version in self.packages.items() if package in
                    extras_require]
        else:
            return [f"{package}{operator.value}{version}" for package, version in self.packages.items()]

    def get_dev_packages(self, operator: OperatorEnum) -> List[str]:
        """
        Get list of dev-packages with correct version and given operator.
        :param operator: (OperatorEnum) e.g. '==' or '>='
        :return: (List[str]) e.g. [package==version,...]
        """
        return [f"{package}{operator.value}{version}" for package, version in self.dev_packages.items()]



from unittest import TestCase

from requirements_manager import RequirementsManager, OperatorEnum
from requirements_manager.errors import NoPackagesInPipfileError, PipfilePathDoesNotExistError


extras_packages = {
    "flask": ["flask", "tensorflow", "boto3", "protobuf", "jinja2", "itsdangerous", "werkzeug",
              "markupsafe"],
    "3d": ["pyvista", "gmsh"],
    "matlab": ["scipy"]
}

flask_packages = ["flask>=1.1.4", "tensorflow>=2.2.0", "boto3>=1.10.5", "protobuf>=3.19.6",
                  "jinja2>=2.11.3", "itsdangerous>=1.1.0", "werkzeug>=1.0.1", "markupsafe>=2.0.1"]
three_d_packages = ["gmsh>=4.9.0", "pyvista>=0.34.2"]
matlab_packages = ["scipy>=1.4.1"]

all_packages = ['globre>=0.1.5', 'h5py>=2.10.0', 'joblib>=0.15.0', 'numpy>=1.16.4', 'pandas>=0.25.1',
                'distributed>=2021.1.1', 'dill>=0.2.9', 'pyyaml>=5.1.2', 'boto3>=1.10.5', 'botocore>=1.13.5',
                'requests>=2.22.0', 'openpyxl>=3.0.7', 'pyarrow>=0.16.0', 'xlwt>=1.3.0', 'xlrd>=1.2.0',
                'dask[complete]>=2020.12.0'] + flask_packages + three_d_packages + matlab_packages


class TestRequirementsManager(TestCase):
    """
    Simple functional Tests for RequirementsManager
    """

    def setUp(self) -> None:
        self.reqs = RequirementsManager(pipfile_loc="tests/resources/Pipfile")

    def test_pipfile_invalid_path(self):
        with self.assertRaises(PipfilePathDoesNotExistError):
            self.reqs = RequirementsManager(pipfile_loc="tests/resources/invalid")

    def test_manager_get_packages_success(self):
        assert set(all_packages) == set(self.reqs.get_packages(operator=OperatorEnum.GREATER_THAN_EQUAL))
        assert set(flask_packages) == set(
            self.reqs.get_packages(operator=OperatorEnum.GREATER_THAN_EQUAL, extras_require=extras_packages["flask"]))
        assert set(three_d_packages) == set(
            self.reqs.get_packages(operator=OperatorEnum.GREATER_THAN_EQUAL, extras_require=extras_packages["3d"]))
        assert set(matlab_packages) == set(
            self.reqs.get_packages(operator=OperatorEnum.GREATER_THAN_EQUAL, extras_require=extras_packages["matlab"]))

    def test_manager_get_packages_fail(self):
        # assert errors raised if 'packages' section of Pipfile contains no packages
        with self.assertRaises(NoPackagesInPipfileError):
            reqs_no_packages = RequirementsManager(pipfile_loc="tests/resources/Pipfile_no_packages")




from unittest import TestCase, main
from unittest.mock import patch

from monolith_filemanager.path import FilePath


class TestFilePath(TestCase):

    @patch("monolith_filemanager.path.FilePath.check_if_s3")
    @patch("monolith_filemanager.path.FilePath.get_file_type")
    def test___init__(self, mock_get_file_type, mock_check_if_s3):
        test = FilePath("this/is/a/path.txt")

        self.assertEqual("path.txt", test.file)
        self.assertEqual("this/is/a/", test.root)
        self.assertEqual(mock_get_file_type.return_value, test.file_type)
        self.assertEqual(mock_check_if_s3.return_value, test.s3)
        mock_get_file_type.assert_called_once_with(file_string="path.txt")

    def test_check_if_s3(self):
        self.assertEqual(True, FilePath.check_if_s3(path_string="s3://test"))
        self.assertEqual(False, FilePath.check_if_s3(path_string="test"))

    def test_get_file_type(self):
        self.assertEqual("txt", FilePath.get_file_type(file_string="test.txt"))
        self.assertIsNone(FilePath.get_file_type(file_string="testtxt"))

    @patch("monolith_filemanager.path.os")
    @patch("monolith_filemanager.path.FilePath.__init__")
    def test_root_exists(self, mock_init, mock_os):
        mock_init.return_value = None
        test = FilePath("test")
        test.root = "test"
        out_come = test.root_exists

        self.assertEqual(mock_os.path.isdir.return_value, out_come)
        mock_os.path.isdir.assert_called_once_with(test.root)

    @patch("monolith_filemanager.path.os")
    @patch("monolith_filemanager.path.FilePath.__init__")
    def test_file_exists(self, mock_init, mock_os):
        mock_init.return_value = None
        test = FilePath("this/is/a/path.txt")

        out_come = test.file_exists

        self.assertEqual(mock_os.path.exists.return_value, out_come)
        mock_os.path.exists.assert_called_once_with(test)


if __name__ == "__main__":
    main()



import tempfile
from typing import Any, Union

from .base import File
from .errors import GmshFileError
from ..path import FilePath


class GmshFile(File):
    """
    This is a class for managing the reading and writing of stl objects.
    """
    SUPPORTED_FORMATS = ['brep', 'stp', 'step', 'igs', 'iges']

    def __init__(self, path: Union[str, FilePath]) -> None:
        """
        The constructor for the CustomDataFile class.

        :param path: (str/FilePath) path to the file
        """
        super().__init__(path=path)

    def read(self, **kwargs) -> Any:
        """
        Gets data from file defined by the file path.

        :return: using tempfile as a buffer to converting a CAD file to a mesh
        """

        try:
            import gmsh
        except ImportError:
            raise GmshFileError(
                message="loading a gmsh file relies on the gmsh module. Please install this module and try again."
            )

        # if we initialize with sys.argv it could be anything
        gmsh.initialize()
        gmsh.option.setNumber("General.Terminal", 1)
        gmsh.model.add('Surface_Mesh_Generation')
        gmsh.open(self.path)

        # create a temporary file for the results
        out_data = tempfile.NamedTemporaryFile(suffix='.stl', delete=False)
        out_data.close()

        return out_data

    def write(self, data: Any, **kwargs) -> None:
        """
        Writes data to file.

        :param data: (python object) data to be written to file
        :return: None
        """

        data.save(self.path)



import pathlib

import setuptools
from setuptools import find_packages
from setuptools.command.build_py import build_py as build_py_orig

from requirements_manager import RequirementsManager, OperatorEnum

# from setuptools import dist
# dist.Distribution().fetch_build_eggs(['Cython==0.29'])
# from Cython.Build import cythonize

extras_packages = {
    "flask": ["flask", "tensorflow", "boto3"],
    "3d": ["pyvista", "gmsh"],
    "matlab": ["scipy"]
}


class CustomBuildPy(build_py_orig):
    """
    subclass build_py so that we collect no .py files inside the built pip package
    this is done by overriding build_packages method with a noop
    """
    def build_packages(self):
        pass


with open("README.md", "r") as fh:
    long_description = fh.read()

with open(str(pathlib.Path(__file__).parent.absolute()) + "/monolith_filemanager/version.py", "r") as fh:
    version = fh.read().split("=")[1].replace("'", "")

directives = {
    'language_level': 3,
    'always_allow_keywords': True
}

requirements = RequirementsManager()


setuptools.setup(
    name="monolith_filemanager",
    version=version,
    author="Maxwell Flitton",
    author_email="maxwell@gmail.com",
    description="Python package for reading and writing files",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/MonolithAILtd/monolith-filemanager",
    install_requires=requirements.get_packages(operator=OperatorEnum.GREATER_THAN_EQUAL),
    extras_require={
        'flask': requirements.get_packages(operator=OperatorEnum.GREATER_THAN_EQUAL, extras_require=extras_packages["flask"]),
        '3d': requirements.get_packages(operator=OperatorEnum.GREATER_THAN_EQUAL, extras_require=extras_packages["3d"]),
        'matlab': requirements.get_packages(operator=OperatorEnum.GREATER_THAN_EQUAL, extras_require=extras_packages["matlab"])
    },
    packages=find_packages(exclude=("tests",)),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3',
    tests_require=['pytest'],
    entry_points={
        'console_scripts': [
            'file-hello = monolith_filemanager.console_commands.hello:print_logo',
            'file-install-flask = monolith_filemanager.console_commands.install_flask:install_flask',
            'file-install-tensorflow = monolith_filemanager.console_commands.install_tensorflow:install_tensorflow',
            'file-install-aws = monolith_filemanager.console_commands.install_boto:install_boto'
        ],
    }
    # ext_modules=cythonize("caching/**/*.py", exclude="tests/**/*.py", compiler_directives=directives, nthreads=4),
    # cmdclass={'build_py': CustomBuildPy},
    # include_package_data=False,
    # options={"bdist_wheel": {"universal": "1"}}
)
